/*
 * Copyright 2009 Serhat Dirik

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
/**
* @author Serhat Dirik  
* This library contains Node and Tree Classes
* Usage sample can be found in registry.html
**/
function Node(_data , _title ,  _icon , _iconOpen){
	if (arguments.length < 2 ||	_title == null ){
	   throw new Error('Wrong arguments');
	}
	if (typeof _title != 'string'){
	  throw new Error('title type must be String');
	}
	if (_icon && typeof _icon != 'string'){
	  throw new Error('icon type must be String');
	}
	if (_iconOpen  && typeof _iconOpen != 'string'){
	  throw new Error('iconOpen type must be String');
	}
	
	var childs = new Array();
	var data         = _data;
	var title        = _title;
	var icon         = _icon;
	var iconOpened   = _iconOpen;
	this.parent      =  null;

	
	function _getData(){
	  return data;
	}
	
    function _getTitle(){
      return title;
    }
    
	function _getIcon(){
	  return _icon;
	}
	
	function _getOpenedIcon(){
	  if (iconOpened == null) return icon ; else return iconOpened;
	}
	
	function _addChild(child,index){ 
	  if (child == null || child.constructor != Node) {
	     throw new Error('Child data type must be Node');
	  }
	  var ret;
	  if (index == null || typeof index != 'number'){
		ret = childs.length + 1;
	    childs[childs.length] = child;
	    
	  }else{
	  	childs[index] = child;
	  	ret = index;
	  }
	  child.parent = this;
	  return ret;
	}
	
	function _removeChild(index){
	   if (index == null || typeof index != 'number'){
		   throw new Error('index data type must be number');
	   }
	   if (childs.length < index) return;
	   childs.splice(index,1);
	}
	
	function _clearChilds(){
	  childs.length = 0;
	}
	
	function _getChild(index){
	   if (index == null || typeof index != 'number'){
		   throw new Error('index data type must be number');
	   }
	   if (childs.length < index) return null;
	   return childs[index - 1];
	}
	
	function _getChilds(){
	  return childs;
	}
	
	function _size(){ return childs.length;}
	
	function _hasChild() { return  _size() > 0 ; } 
	
	this.getTitle    = _getTitle;
	this.addChild    = _addChild;
	this.removeChild = _removeChild;
	this.clearChilds = _clearChilds;
	this.getChild    = _getChild;
	this.getChilds   = _getChilds;
	this.size        = _size;
	this.getIcon     = _getIcon;
	this.getOpenedIcon = _getOpenedIcon;
	this.getTitle    = _getTitle;
	this.getData   = _getData;
	this.hasChild = _hasChild;
}

//////////////////////////////////////////////////
//TREE CLASS
//////////////////////////////////////////////

function Tree(_containerID , _title , _isOpened , _icon , _iconOpen){
    if (arguments.length < 5 ||	_title == null || _isOpened == null || ! _containerID){
	   throw new Error('Wrong arguments');
	}
	if (typeof _title != 'string'){
	  throw new Error('title type must be String');
	}
	if (typeof _icon != 'string'){
	  throw new Error('icon type must be String');
	}
	if (typeof _iconOpen != 'string'){
	  throw new Error('iconOpen type must be String');
	}
	
	var container = document.getElementById(_containerID);
	if (! container || container == null || !container.tagName || container.tagName != 'DIV') {
	   throw new Error('Container id must indicate a DIV element');
	}
	var self = this;
	var root = new Node(null,_title,_icon,_iconOpen);
	root.isOpened = _isOpened;
	var selectedElement   = null;
	var textColor         = '#000000';
	var backgroundColor	  = '#ffffff';
	var lineColor         = "#cccccc"; 
	var selectedColor	  = '#99ccff';
	var selectedTextColor = '#cc0000';
	var pointColor		  = '#ffffcc';
	var pointTextColor	  = '#ff0000';
	
	this.onNodeSelect = null;
	this.onNodeDeselect =  null;
	this.onNodeExpand = null;
	this.onNodeCollapse = null;
	_draw();
	
	function _addRoot(node){
	   var index= root.addChild(node);
	   node.level=1;
	   root.elem.appendChild(_prepNode(node));
	   //_draw();
	   return index;
	}
	
	function _getRoot(index){
	  return root.getChild(index);
	}
	
	function _removeRoot(index){
	  if (index > root.size() || index <= 0) return;
	  root.elem.removeChild(root.getChild(index).elem);
	  root.removeChild(index);
	  //_draw();
	}
	function _rootSize(){
	  return root.size();
	}
	
	function _clearRoots(){
	   root.clearChilds();
	  _draw();
	}
	
	function _getSelectedElement(){
	  return selectedElement;
	}
	
	function _getSelectedNode(){
	  return ( _getSelectedElement() != null ? _getSelectedElement().node : null) ;
	}
	
	function _draw(){
		while (container.firstChild){
		  container.removeChild(container.firstChild);
		}
		//container.style.fontFamily="Verdana, Geneva, Arial, Helvetica, sans-serif";
		container.style.fontFamily="'Courier New', sans-serif";
		container.style.fontSize="14px";
		container.style.whiteSpace="nowrap";
		container.style.backgroundColor = backgroundColor;
		container.style.color = textColor;
		container.appendChild( _prepNode(root));
		//return str;
	}
   

   
   function _prepNode(node ){
     if (node.isOpened == null) node.isOpened = false;
     var elem = document.createElement('div');
     elem.img = null;
     elem.title=null;
     //Put Indentation
     var indent = 0;
     if (node.level || node.level != null) {
        indent=node.level*1;
     }else{
        node.level =0;
     }
     var str="";
     var nbsp = String.fromCharCode(160);
     for (var i = 0 ; i < indent ; i++){
       str += nbsp + nbsp +  nbsp ;
       
     }
     if (node.parent && node.parent != null) {
	    // elem.appendChild(document.createTextNode(str+"|_"));   
	     elem.appendChild(document.createTextNode(str));   
     }
     //Put Indicator
     if (node.hasChild() || node == root){
	      var sig = document.createElement('a');
		  sig.style.cursor="pointer";
		  sig.style.fontWeight="bolder";
		  sig.style.border = "solid "+lineColor+" 1px";
	      sig.onclick = _clickedOnInd;
		  sig.node = node;
		  sig.container = elem;
	     if (node.isOpened){
		     sig.appendChild(document.createTextNode('-'));
	     }else{
		     sig.appendChild(document.createTextNode('+'));
	     }
	     elem.appendChild(sig);
	 }     
	 //Put Icon
	 if (node.getIcon() != null){
	    var img = document.createElement('img');
	    img.node = node;
	    img.container = elem;
	    img.style.border="0px";
	    img.onmouseover = _imgOnMouseOver;
	    img.onmouseout  = _imgOnMouseOut;
     	img.onclick = _nodeMouseClicked;
		img.style.verticalAlign="middle";
		if (node.isOpened){
			img.src = node.getOpenedIcon();
		}else{
			img.src = node.getIcon();
		}
		elem.img = img;
		elem.appendChild(img);
	 }
	 //PUT Title
	 var title = document.createElement('a');
	 title.style.cursor='default';
	 title.appendChild(document.createTextNode(node.getTitle()));
	 title.node = node;
	 title.container = elem;
	 title.onmouseover = _titleOnMouseOver;
	 title.onmouseout  = _titleOnMouseOut;
	 title.onclick = _nodeMouseClicked;
	 elem.titleText = title;
	 elem.appendChild(title);
	 //Show hide this part
	 if (node.parent && node.parent != null){
	   if(node.parent.isOpened && node.parent.isOpened == true){
	   		elem.style.display = "block";
	   }else{
	        elem.style.display = "none";
	   }
	 }else{
	   elem.display = "block";
	 }
	 //Prep childs
	 if (node.size() > 0) {
	     indent++;
	     var childs = node.getChilds();
	     for (var i = 0 ; i < childs.length;i++){
	       childs[i].level=indent;
	       elem.appendChild(_prepNode(childs[i]));
	     }
	 }
     node.elem = elem;
     elem.node = node;
     return elem;
   }
   
      function _clickedOnInd(){

     	var loccontainer = this.container;
     	var locnode  = this.node;
 		if(locnode.isOpened){
 			if ( self && self.onNodeCollapse != null) self.onNodeCollapse(loccontainer);
 		}else{
 			if ( self && self.onNodeExpand != null) self.onNodeExpand(loccontainer);
 		}
     	
     	locnode.isOpened = ! locnode.isOpened;
     	//Open close childs
     	for (var i = 0 ; i < loccontainer.childNodes.length ; i++){
     	  var elem = loccontainer.childNodes[i];
     	  if ( elem.tagName != 'DIV') continue;
     	  if (locnode.isOpened){
     	  		elem.style.display = "block";
		   }else{
		        elem.style.display = "none";
		   }
     	}
     	//Replace Indicator
     	while (this.firstChild){
		  this.removeChild(this.firstChild);
		}
		
     	if (locnode.isOpened){
		     this.appendChild(document.createTextNode('-'));
		     if (loccontainer.img != null){
     	  		  loccontainer.img.src = locnode.getOpenedIcon();
     	  	  }
	     }else{
		     this.appendChild(document.createTextNode('+'));
		     if (loccontainer.img != null){
     	  		  loccontainer.img.src = locnode.getIcon();
     	  	  }
	     }

   	  }  
   	  
   	  function _imgOnMouseOver(){
		 if (this.node.isOpened == false && this.node.getOpenedIcon() != null) {
	   	     this.src = this.node.getOpenedIcon();
	   	 }
 	   	 if (selectedElement == this.container) return;
	   	 this.container.titleText.style.backgroundColor = pointColor;

   	  }
   	  
   	  function _imgOnMouseOut(){
   	  
		 if (this.node.isOpened == false && this.node.getIcon() != null) {
	   	     this.src = this.node.getIcon();
	   	 }
	     if (selectedElement == this.container) return; 	 
	   	 this.container.titleText.style.backgroundColor = backgroundColor;
   	  }
   	  
   	  function _titleOnMouseOver(){
	   	 this.style.color           = pointTextColor;
   	     if (selectedElement == this.container) return;
		 this.style.backgroundColor = pointColor;

   	  }
   	  
   	  function _titleOnMouseOut(){
 	   	  
   	      if (selectedElement == this.container){
	   	      this.style.color           = selectedTextColor;
   	         return;
   	      }
		 this.style.color           = textColor;
	   	 this.style.backgroundColor = backgroundColor;
   	  }
   	  
   	 function _nodeMouseClicked(){
   	 	if (selectedElement == this.container){
	   	 	_deselectElement(this.container);
	   	 	return ;
   	 	}
   	 	if (selectedElement != null){
   	 		_deselectElement(selectedElement);
   	 	}
   	 	_selectElement(this.container);
   	 }
   	  	 
   	  function _selectElement(elem){
		elem.titleText.style.backgroundColor = selectedColor;
		elem.titleText.style.color = selectedTextColor;
		selectedElement = elem;
		if ( self.onNodeSelect != null) self.onNodeSelect(elem);
   	  }
   	  
   	  function _deselectElement(elem){
   	  	elem.titleText.style.backgroundColor = backgroundColor;
   	  	elem.titleText.style.color = textColor;
   	  	if (selectedElement == elem){
   	  		 selectedElement = null;
	   	  	if ( self.onNodeDeselect != null) self.onNodeDeselect(elem);
	   	}
   	  }
   	  
  	this.reDraw = _draw;
	this.draw     = _draw;
	this.addRoot  = _addRoot;
	this.getRoot  = _getRoot;
	this.removeRoot = _removeRoot;
	this.rootSize = _rootSize;
	this.clearRoots = _clearRoots;
	this.getSelectedElement = _getSelectedElement;
	this.getSelectedNode    = _getSelectedNode;
}